package info.bukova.isspst.services.settings;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.Unmarshaller;
import org.exolab.castor.xml.ValidationException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;

import info.bukova.isspst.data.Address;
import info.bukova.isspst.data.GlobalSettings;
import info.bukova.isspst.data.SettingsData;
import info.bukova.isspst.services.AbstractOwnedService;
import info.bukova.isspst.services.IsspstException;

public class GlobalSettingServiceImpl extends AbstractOwnedService<GlobalSettings> implements
		GlobalSettingsService {
	
	private final static Logger log = LoggerFactory.getLogger(GlobalSettingsService.class);
	private final static String MARSHAL_ERROR = "Cannot marshal settings data: ";
	private final static String UNMARSHAL_ERROR = "Cannot unmarshal settings data: ";

	private Marshaller marshaller;
	private Unmarshaller unmarshaller;
	private SettingsData settings;
	
	public GlobalSettingServiceImpl(Marshaller marshaller, Unmarshaller unmarshaller) {
		this.marshaller = marshaller;
		this.unmarshaller = unmarshaller;
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_ADD')")
	public void add(GlobalSettings entity) {
		if (!this.getAll().isEmpty()) {
			throw new IsspstException("Global settings allready exists");
		}
		
		if (entity.getData() == null || entity.getData().isEmpty()) {
			SettingsData data = new SettingsData();
			entity.setData(marshalData(data));
		}
		
		super.add(entity);
	}
	
	private String marshalData(SettingsData data) {
		StringWriter wr = new StringWriter();
		try {
			marshaller.setWriter(wr);
			marshaller. marshal(data);
		} catch (MarshalException e) {
			log.error(MARSHAL_ERROR + e.getMessage());
		} catch (ValidationException e) {
			log.error(MARSHAL_ERROR + e.getMessage());
		} catch (IOException e) {
			log.error(MARSHAL_ERROR + e.getMessage());
		}
		
		return wr.toString();
		
	}
	
	private SettingsData unmarshalData(String data) {
		StringReader sr = new StringReader(data);
		try {
			unmarshaller.setClass(SettingsData.class);
			return (SettingsData) unmarshaller.unmarshal(sr);
		} catch (MarshalException e) {
			log.error(UNMARSHAL_ERROR + e.getMessage());
		} catch (ValidationException e) {
			log.error(UNMARSHAL_ERROR + e.getMessage());
		}
		
		return null;
	}

	@Override
	@Transactional
	public SettingsData getSettings() {
		if (settings == null) {
			GlobalSettings gs = this.getAll().get(0);
			settings = unmarshalData(gs.getData());
		}
		return settings;
	}

	@Override
	@Transactional
	@PreAuthorize("hasPermission(this, 'PERM_EDIT')")
	public void updateSettings() {
		if (this.getAll().isEmpty()) {
			throw new IsspstException("Global settings does not exists");
		}
		
		GlobalSettings gs = this.getAll().get(0);
		
		for (Address a : settings.getShippingAddrs()) {
			a.setCompany(settings.getMainAddress().getCompany());
			a.setIc(settings.getMainAddress().getIc());
			a.setDic(settings.getMainAddress().getDic());
		}
		
		gs.setData(marshalData(settings));
		
		super.update(gs);
	}
	
	
	
}
